@@ -32,7 +32,7 @@ from peewee import DateField
 from peewee import DateTimeField
 from peewee import ForeignKeyField
 from peewee import TextField
-from werkzeug import Headers
+from werkzeug.datastructures import Headers
 from wtforms import fields
 from wtforms import widgets
 from wtfpeewee.fields import ModelHiddenField
@@ -75,6 +75,20 @@ class AdminFilterModelConverter(FilterModelConverter):
         return field.name, form_field
 
 
+class Action(object):
+    def __init__(self, name=None, description=None):
+        self.name = name or (type(self).__name__.replace('Action', ''))
+        self.description = description or re.sub('[\-_]', ' ', self.name).title()
+
+    def callback(self, id_list):
+        """
+        Perform an action on the list of IDs specified. If the return value is
+        a Response object, then that will be returned to the user. Otherwise,
+        the return value is ignored and the user is redirected to the index.
+        """
+        raise NotImplementedError
+
+
 class ModelAdmin(object):
     """
     ModelAdmin provides create/edit/delete functionality for a peewee Model.
@@ -97,6 +111,9 @@ class ModelAdmin(object):
 
     form_converter = AdminModelConverter
 
+    # User-defined bulk actions. List or tuple of Action instances.
+    actions = None
+
     # foreign_key_field --> related field to search on, e.g. {'user': 'username'}
     foreign_key_lookups = None
 
@@ -125,6 +142,9 @@ class ModelAdmin(object):
         self.templates = dict(self.base_templates)
         self.templates.update(self.get_template_overrides())
 
+        self.action_map = dict((action.name, action)
+                               for action in (self.actions or ()))
+
     def get_template_overrides(self):
         return {}
 
@@ -209,9 +229,28 @@ class ModelAdmin(object):
         return {}
 
     def index(self):
+        if request.method == 'POST':
+            id_list = request.form.getlist('id')
+            action = request.form['action']
+            if action == 'delete':
+                return redirect(url_for(self.get_url_name('delete'), id=id_list))
+            elif action == 'export':
+                return redirect(url_for(self.get_url_name('export'), id=id_list))
+            elif action in self.action_map:
+                id_list = request.form.getlist('id')
+                if not id_list:
+                    flash('Please select one or more rows.', 'warning')
+                else:
+                    action_obj = self.action_map[action]
+                    maybe_response = action_obj.callback(id_list)
+                    if isinstance(maybe_response, Response):
+                        return maybe_response
+            else:
+                flash('Unknown action: "%s".' % action, 'danger')
+            return self._index_redirect()
+
         session['%s.index' % self.get_admin_name()] = request.url
         query = self.get_query()
-
         ordering = request.args.get('ordering') or ''
         query = self.apply_ordering(query, ordering)
 
@@ -221,13 +260,6 @@ class ModelAdmin(object):
         # create a paginated query out of our filtered results
         pq = PaginatedQuery(query, self.paginate_by)
 
-        if request.method == 'POST':
-            id_list = request.form.getlist('id')
-            if request.form['action'] == 'delete':
-                return redirect(url_for(self.get_url_name('delete'), id=id_list))
-            else:
-                return redirect(url_for(self.get_url_name('export'), id=id_list))
-
         return render_template(self.templates['index'],
             model_admin=self,
             query=pq,
@@ -238,16 +270,19 @@ class ModelAdmin(object):
             **self.get_extra_context()
         )
 
+    def _index_redirect(self):
+        url = (session.get('%s.index' % self.get_admin_name()) or
+               url_for(self.get_url_name('index')))
+        return redirect(url)
+
     def dispatch_save_redirect(self, instance):
         if 'save' in request.form:
-            url = (session.get('%s.index' % self.get_admin_name()) or
-                   url_for(self.get_url_name('index')))
-            return redirect(url)
+            return self._index_redirect()
         elif 'save_add' in request.form:
             return redirect(url_for(self.get_url_name('add')))
         else:
             return redirect(
-                url_for(self.get_url_name('edit'), pk=instance.get_id())
+                url_for(self.get_url_name('edit'), pk=instance._pk)
             )
 
     def add(self):
@@ -300,10 +335,10 @@ class ModelAdmin(object):
 
         for query, fk in obj.dependencies():
             if not fk.null:
-                sq = fk.model_class.select().where(query)
+                sq = fk.model.select().where(query)
                 collected = [rel_obj for rel_obj in sq.execute().iterator()]
                 if collected:
-                    objects.append((0, fk.model_class, collected))
+                    objects.append((0, fk.model, collected))
 
         return sorted(objects, key=lambda i: (i[0], i[1].__name__))
 
@@ -319,7 +354,7 @@ class ModelAdmin(object):
             collected = {}
             if self.delete_collect_objects:
                 for obj in query:
-                    collected[obj.get_id()] = self.collect_objects(obj)
+                    collected[obj._pk] = self.collect_objects(obj)
 
         elif request.method == 'POST':
             count = query.count()
@@ -327,9 +362,7 @@ class ModelAdmin(object):
                 obj.delete_instance(recursive=self.delete_recursive)
 
             flash('Successfully deleted %s %ss' % (count, self.get_display_name()), 'success')
-            url = (session.get('%s.index' % self.get_admin_name()) or
-                   url_for(self.get_url_name('index')))
-            return redirect(url)
+            return self._index_redirect()
 
         return render_template(self.templates['delete'], **dict(
             model_admin=self,
@@ -373,7 +406,7 @@ class ModelAdmin(object):
 
         return render_template(self.templates['export'],
             model_admin=self,
-            model=query.model_class,
+            model=query.model,
             query=query,
             filter_form=filter_form,
             field_tree=field_tree,
@@ -414,7 +447,7 @@ class ModelAdmin(object):
             if field.null:
                 data.append({'id': '__None', 'repr': 'None'})
 
-            data.extend([{'id': obj.get_id(), 'repr': unicode(obj)} for obj in pq.get_list()])
+            data.extend([{'id': obj._pk, 'repr': str(obj)} for obj in pq.get_list()])
 
         json_data = json.dumps({'prev_page': prev_page, 'next_page': next_page, 'object_list': data})
         return Response(json_data, mimetype='application/json')
@@ -492,7 +525,7 @@ class AdminTemplateHelper(object):
         except KeyError:
             return self.fix_underscores(column_name)
         else:
-            return field.verbose_name
+            return field.verbose_name or self.fix_underscores(field.name)
 
     def get_model_admins(self):
         return {'model_admins': self.admin.get_model_admins(), 'branding': self.admin.branding}
@@ -500,7 +533,7 @@ class AdminTemplateHelper(object):
     def get_admin_url(self, obj):
         model_admin = self.admin.get_admin_for(type(obj))
         if model_admin:
-            return url_for(model_admin.get_url_name('edit'), pk=obj.get_id())
+            return url_for(model_admin.get_url_name('edit'), pk=obj._pk)
 
     def get_model_name(self, model_class):
         model_admin = self.admin.get_admin_for(model_class)
@@ -669,7 +702,7 @@ class Export(object):
         def ensure_join(query, m, p):
             if m not in joined:
                 if '__' not in p:
-                    next_model = query.model_class
+                    next_model = query.model
                 else:
                     next, _ = p.rsplit('__', 1)
                     next_model = self.alias_to_model[next]
@@ -687,7 +720,7 @@ class Export(object):
                 model = self.alias_to_model[path]
                 clone = ensure_join(clone, model, path)
             else:
-                model = self.query.model_class
+                model = self.query.model
                 column = lookup
 
             field = model._meta.fields[column]
@@ -701,18 +734,19 @@ class Export(object):
         prepared_query = self.prepare_query()
         field_dict = {}
         for field in prepared_query._select:
-            field_dict.setdefault(field.model_class, [])
-            field_dict[field.model_class].append(field.name)
+            field_dict.setdefault(field.model, [])
+            field_dict[field.model].append(field.name)
 
         def generate():
             i = prepared_query.count()
-            yield '[\n'
+            yield b'[\n'
             for obj in prepared_query:
                 i -= 1
-                yield json.dumps(serializer.serialize_object(obj, field_dict))
+                obj_data = serializer.serialize_object(obj, field_dict)
+                yield json.dumps(obj_data).encode('utf-8')
                 if i > 0:
-                    yield ',\n'
-            yield '\n]'
+                    yield b',\n'
+            yield b'\n]'
         headers = Headers()
         headers.add('Content-Type', 'application/javascript')
         headers.add('Content-Disposition', 'attachment; filename=%s' % filename)
