@@ -2,14 +2,16 @@ import datetime
 import operator
 
 from flask import request
-from flask_peewee.forms import BaseModelConverter
-from flask_peewee._compat import reduce
 from peewee import *
 from wtforms import fields
 from wtforms import form
 from wtforms import validators
 from wtforms import widgets
 
+from flask_peewee.forms import BaseModelConverter
+from flask_peewee._compat import reduce
+from flask_peewee._wtforms_compat import TextField as WTextField
+
 
 class QueryFilter(object):
     """
@@ -162,7 +164,7 @@ class FilterMapping(object):
             DoubleField: 'numeric',
             DecimalField: 'numeric',
             BooleanField: 'boolean',
-            PrimaryKeyField: 'numeric',
+            AutoField: 'numeric',
             ForeignKeyField: 'foreign_key',
         }
 
@@ -291,7 +293,7 @@ class FilterForm(object):
         return field.default
 
     def get_value_field(self, field):
-        field_name, form_field = self.model_converter.convert(field.model_class, field, None)
+        field_name, form_field = self.model_converter.convert(field.model, field, None)
 
         form_field.kwargs['default'] = self.get_field_default(field)
         form_field.kwargs['validators'] = [validators.Optional()]
@@ -350,7 +352,7 @@ class FilterForm(object):
             for child_prefix, child in node.children.items():
                 new_prefix = prefix + self.field_relation_prefix + child_prefix + self.separator
                 model_copy = list(models) + [child.model]
-                join_copy = list(join_columns) + [child_prefix]
+                join_copy = list(join_columns) + [node.model._meta.fields[child_prefix]]
                 _dfs(child, new_prefix, model_copy, join_copy)
 
         _dfs(self._field_tree, '', [], [])
@@ -387,5 +389,5 @@ class FilterModelConverter(BaseModelConverter):
     def __init__(self, *args, **kwargs):
         super(FilterModelConverter, self).__init__(*args, **kwargs)
         self.defaults = dict(self.defaults)
-        self.defaults[TextField] = fields.TextField
+        self.defaults[TextField] = WTextField
         self.defaults[DateTimeField] = fields.DateTimeField
