@@ -117,6 +117,9 @@ class AdminAuthentication(UserAuthentication):
 
 class RestResource(object):
     paginate_by = 20
+    value_transforms = {'False': False, 'false': False,
+                        'True': True, 'true': True,
+                        'None': None, 'none': None}
 
     # serializing: dictionary of model -> field names to restrict output
     fields = None
@@ -219,13 +222,17 @@ class RestResource(object):
                 filter_expr = '%s%s' % (prefix, field.name)
                 if filter_expr in raw_filters:
                     for op, arg_list, negated in raw_filters[filter_expr]:
-                        query = self.apply_filter(query, filter_expr, op, arg_list, negated)
+                        clean_args = self.clean_arg_list(arg_list)
+                        query = self.apply_filter(query, filter_expr, op, clean_args, negated)
 
             for child_prefix, child_node in node.children.items():
                 queue.append((child_node, prefix + child_prefix + '__'))
 
         return query
 
+    def clean_arg_list(self, arg_list):
+        return [self.value_transforms.get(arg, arg) for arg in arg_list]
+
     def apply_filter(self, query, expr, op, arg_list, negated):
         query_expr = '%s__%s' % (expr, op)
         constructor = lambda kwargs: negated and ~DQ(**kwargs) or DQ(**kwargs)
@@ -281,8 +288,7 @@ class RestResource(object):
         return Response('Bad request', 400)
 
     def response(self, data):
-        kwargs = {} if request.is_xhr else {'indent': 2}
-        return Response(json.dumps(data, **kwargs), mimetype='application/json')
+        return Response(json.dumps(data), mimetype='application/json')
 
     def require_method(self, func, methods):
         @functools.wraps(func)
@@ -419,8 +425,12 @@ class RestResource(object):
                 setattr(instance, k, rel_resource.save_object(rel_obj, v))
 
     def read_request_data(self):
-        data = request.data or request.form.get('data') or ''
-        return json.loads(data.decode('utf8'))
+        if request.data:
+            return json.loads(request.data.decode('utf-8'))
+        elif request.form.get('data'):
+            return json.loads(request.form['data'])
+        else:
+            return dict(request.form)
 
     def create(self):
         try:
